@@ -315,91 +315,91 @@ otherprog:
 		uid = getuid();
 		truid = runastrusteduser();
 		if ((mfp = mail_open(MSG_RFC822)) == NULL) {
-			n = errno;
-			fprintf(stderr, "%s: cannot submit mail (err# %d)!\n",
-					zmailer, errno);
+		  n = errno;
+		  fprintf(stderr, "%s: cannot submit mail (err# %d)!\n",
+			  zmailer, errno);
 #ifdef	LOG_CRIT
 #ifdef	LOG_MAIL
-			openlog("sendmail", LOG_PID, LOG_MAIL);
-#else	/* !LOG_MAIL */
-			openlog("sendmail", LOG_PID);
-#endif	/* LOG_MAIL */
-			errno = n;
-			syslog(
+		  openlog("sendmail", LOG_PID, LOG_MAIL);
+#else  /* !LOG_MAIL */
+		  openlog("sendmail", LOG_PID);
+#endif /* LOG_MAIL */
+		  errno = n;
+		  syslog(
 #ifdef	LOG_SALERT
-			       LOG_SALERT
-#else	/* LOG_SALERT */
-			       LOG_ALERT
-#endif	/* LOG_SALERT */
-			       , "cannot submit mail for uid %d: %m", uid);
-#endif	/* LOG_CRIT */
-			mfp = deadletter(uid);
+			 LOG_SALERT
+#else  /* LOG_SALERT */
+			 LOG_ALERT
+#endif /* LOG_SALERT */
+			 , "cannot submit mail for uid %d: %m", uid);
+#endif /* LOG_CRIT */
+		  mfp = deadletter(uid);
 		}
 		if (truid != uid) {
-			/* if we are setuid root, and not invoked by root,
-			   hardwire in the definition of from so the mailer
-			   will know */
+		  /* if we are setuid root, and not invoked by root,
+		     hardwire in the definition of from so the mailer
+		     will know */
 #ifdef	USE_FCHOWN
-			runasrootuser();
-			if (uid != 0 && fchown(fileno(mfp), uid, -1) == 0)
-				;
-			else
-#endif	/* USE_FCHOWN */
-			if (from == NULL) {
-				if (uid != 0 && (pwd = getpwuid(uid)) != NULL)
-					from = pwd->pw_name;
-				else if ((cp = getlogin()) != NULL)
-					from = cp;
-				else /* could find with getpwnam */
-					from = "root";
-			}
+		  runasrootuser();
+		  if (uid != 0 && fchown(fileno(mfp), uid, -1) == 0)
+		    ;
+		  else
+#endif /* USE_FCHOWN */
+		    if (from == NULL) {
+		      if (uid != 0 && (pwd = getpwuid(uid)) != NULL)
+			from = pwd->pw_name;
+		      else if ((cp = getlogin()) != NULL)
+			from = cp;
+		      else	/* could find with getpwnam */
+			from = "root";
+		    }
 #ifdef	USE_FCHOWN
-			runastrusteduser();
-#endif	/* USE_FCHOWN */
+		  runastrusteduser();
+#endif /* USE_FCHOWN */
 		}
 
 		vfd = -1;
 		if (verbose) {
-			if (postoffice == NULL) {
-				postoffice = getzenv("POSTOFFICE");
-				if (postoffice == NULL)
-					postoffice = POSTOFFICE;
-			}
-			verbfile = emalloc(strlen(postoffice)
-						 +strlen(PUBLICDIR)+20);
-			sprintf(verbfile, "%s/%s/v_XXXXXX",
-					  postoffice, PUBLICDIR);
-			mktemp(verbfile);
-			if (*verbfile == '\0' ||
-			    (vfd = open(verbfile, O_CREAT|O_RDWR, 0600)) < 0) {
-				fprintf(stderr,
-				"%s: cannot create verbose log file in %s/%s\n",
-						zmailer, postoffice, PUBLICDIR);
-				verbfile = NULL;
-			} else {
-				/*
-				 * We need to make it a relative pathname
-				 * in case the router/scheduler's idea of
-				 * root directory is different than ours.
-				 */
-				cp = strrchr(verbfile, '/');
-				while (--cp > verbfile)
-					if (*cp == '/')
-						break;
-				fprintf(mfp, "verbose \"../%s\"\n", cp+1);
-			}
+		  if (postoffice == NULL) {
+		    postoffice = getzenv("POSTOFFICE");
+		    if (postoffice == NULL)
+		      postoffice = POSTOFFICE;
+		  }
+		  verbfile = emalloc(strlen(postoffice)
+				     +strlen(PUBLICDIR)+20);
+		  sprintf(verbfile, "%s/%s/v_XXXXXX",
+			  postoffice, PUBLICDIR);
+		  mktemp(verbfile);
+		  if (*verbfile == '\0' ||
+		      (vfd = open(verbfile, O_CREAT|O_RDWR, 0600)) < 0) {
+		    fprintf(stderr,
+			    "%s: cannot create verbose log file in %s/%s\n",
+			    zmailer, postoffice, PUBLICDIR);
+		    verbfile = NULL;
+		  } else {
+		    /*
+		     * We need to make it a relative pathname
+		     * in case the router/scheduler's idea of
+		     * root directory is different than ours.
+		     */
+		    cp = strrchr(verbfile, '/');
+		    while (--cp > verbfile)
+		      if (*cp == '/')
+			break;
+		    fprintf(mfp, "verbose \"../%s\"\n", cp+1);
+		  }
 		}
 		if (external)
-			fprintf(mfp, "external\n");
+		  fprintf(mfp, "external\n");
 		if (fullname != NULL && *fullname != '\0')
-			/* maybe we should put it in the environment? */
-			fprintf(mfp, "fullname %s\n", fullname);
+		  /* maybe we should put it in the environment? */
+		  fprintf(mfp, "fullname %s\n", fullname);
 		if (from != NULL && *from != '\0')
-			fprintf(mfp, "from %s\n", from);
+		  fprintf(mfp, "from %s\n", from);
 		if (!binary) {
-			for (; optind < argc; ++optind)
-				/*fprintf(mfp, "to %s\n", argv[optind]);*/
-				check_and_print_to(mfp,argv[optind]);
+		  for (; optind < argc; ++optind)
+		    /*fprintf(mfp, "to %s\n", argv[optind]);*/
+		    check_and_print_to(mfp,argv[optind]);
 		}
 
 		if (fflush(mfp) == EOF
@@ -407,103 +407,134 @@ otherprog:
 		    || fsync(fileno(mfp)) < 0
 /* #endif */	/* NFSFSYNC */
 		    || ferror(mfp)) {
-			mail_abort(mfp);
-			mfp = deadletter(uid);
-			errflg = 1;
+		  mail_abort(mfp);
+		  mfp = deadletter(uid);
+		  errflg = 1;
 		}
 
 		/*
 		 * The postoffice cannot be changed from outside mail_open(),
 		 * which, though it may be unsatisfactory, is good for security.
 		 */
-
-		if (binary) {  /* haa@cs.hut.fi  found a bug in read() --
-				  sizeof buf[0] != sizeof buf  :-)     */
-			while ((n = read(0, buf, sizeof buf)) > 0) {
-				if (fwrite(buf, sizeof buf[0], n, mfp) != n)
-					break;
-			}
+		
+		if (binary) {	/* haa@cs.hut.fi  found a bug in read() --
+				   sizeof buf[0] != sizeof buf  :-)     */
+		  while ((n = read(0, buf, sizeof buf)) > 0) {
+		    if (fwrite(buf, sizeof buf[0], n, mfp) != n)
+		      break;
+		  }
 		} else {
-			/* cmd line can only set dotiseof to 0.. improvement? */
-			if (dotiseof)
-				dotiseof = isatty(fileno(stdin));
-			n = 0;
-			while (fgets(buf, sizeof buf, stdin)) {
-				if (++n == 1) {
-					if (strncmp(buf,"From ",5)==0) {
-						/* [mea@utu.fi] I vote for
-						   removing this line if next
-						   is a RFC-header */
-						buf2 = emalloc(2+strlen(buf));
-						if (!buf2) { errflg=1; break; }
-						strcpy(buf2,buf);
-						continue;
-					}
-					for (s = buf;
-					     isascii(*s) && !isspace(*s)
-					         && *s != ':';
-					     ++s)
-						continue;
-					if (*s != ':')
-						putc('\n', mfp);
-				}
-				if (n==2 && buf2) { /* Handle 2nd line if first begun
-						       with a "From "  */
-					for (s = buf;
-					     isascii(*s) && !isspace(*s) && *s != ':';
-					     ++s)
-						continue;
-					if (*s != ':') {
-						putc('\n', mfp);
-						fputs( buf2, mfp );
-					}
-				}
-				s = buf;
-				if (dotiseof && *s == '.' && *++s == '\n')
-					break;
-				fputs(buf, mfp);
-			}
+		  /* cmd line can only set dotiseof to 0.. improvement? */
+		  int crlf_strip = 0;
+		  if (dotiseof)
+		    dotiseof = isatty(fileno(stdin));
+		  n = 0;
+		  while (fgets(buf, sizeof buf, stdin)) {
+		    /* In case the input contains CRLF instead of LF,
+		       do convert them.. */
+		    if (crlf_strip) {
+		      s = strrchr(buf, '\r');
+		      if (s && s[1] == '\n' && s[2] == 0) {
+			s[0] = '\n';
+			s[1] = 0;
+		      }
+		    }
+		    if (++n == 1) {
+
+		      /* The first line, see if we have junk in the begin..
+			 But detect at first the input for CRLF line
+			 termination in place of the UNIX-normal LF ... */
+
+		      if ((s = strrchr(buf, '\r')) != NULL &&
+			  s[1] == '\n' && s[2] == 0) {
+			/* Brr... CRLF+NULL */
+			crlf_strip = 1;
+			s[0] = '\n';
+			s[1] = 0;
+		      }
+		      if (strncmp(buf,"From ",5)==0) {
+			/* [mea@utu.fi] I vote for
+			   removing this line if next
+			   is a RFC-header */
+			buf2 = emalloc(2+strlen(buf));
+			if (!buf2) { errflg=1; break; }
+			strcpy(buf2,buf);
+			continue;
+		      }
+		      if (strncmp(buf,">From ",6)==0) {
+			/* [mea@utu.fi] I vote for
+			   removing this line if next
+			   is a RFC-header */
+			buf2 = emalloc(2+strlen(buf));
+			if (!buf2) { errflg=1; break; }
+			strcpy(buf2,buf);
+			continue;
+		      }
+		      for (s = buf;
+			   isascii(*s) && !isspace(*s)
+			   && *s != ':';
+			   ++s)
+			continue;
+		      if (*s != ':')
+			putc('\n', mfp);
+		    }
+		    if (n==2 && buf2) { /* Handle 2nd line if first begun
+					   with a "From "  */
+		      for (s = buf;
+			   isascii(*s) && !isspace(*s) && *s != ':';
+			   ++s)
+			continue;
+		      if (*s != ':') {
+			putc('\n', mfp);
+			fputs( buf2, mfp );
+		      }
+		    }
+		    s = buf;
+		    if (dotiseof && *s == '.' && *++s == '\n')
+		      break;
+		    fputs(buf, mfp);
+		  }
 		}
 
 		fflush(mfp);
 		if (errflg || ferror(mfp)) {
-			fprintf(stderr,
-				"%s: message not submitted due to I/O error!\n",
-				zmailer);
-			if (!errflg)
-				mail_abort(mfp);
-			if (vfd >= 0)
-				unlink(verbfile);
-			exit(EX_IOERR);
+		  fprintf(stderr,
+			  "%s: message not submitted due to I/O error!\n",
+			  zmailer);
+		  if (!errflg)
+		    mail_abort(mfp);
+		  if (vfd >= 0)
+		    unlink(verbfile);
+		  exit(EX_IOERR);
 		} else if (mail_close(mfp) == EOF) {
-			fprintf(stderr, "%s: message not submitted!\n",
-					zmailer);
-			if (vfd >= 0)
-				unlink(verbfile);
-			exit(EX_UNAVAILABLE);
+		  fprintf(stderr, "%s: message not submitted!\n",
+			  zmailer);
+		  if (vfd >= 0)
+		    unlink(verbfile);
+		  exit(EX_UNAVAILABLE);
 		}
 		if (vfd >= 0) {
-			if (fork() > 0) exit(EX_OK); /* Let the child to do
-							the trace printout */
-			if (vfd != 0) close(0);
-			if (vfd != 1) close(1);
-			vfp = fdopen(vfd, "r");
-			while (vfp != NULL) {
-				while (fgets(buf, sizeof buf, vfp) != NULL)
-					fprintf(stderr, "%s", buf);
-				if (strncmp(buf, "scheduler done", 14) == 0)
-					break;
-				clearerr(vfp);
-				sleep(1);
-			}
+		  if (fork() > 0) exit(EX_OK); /* Let the child to do
+						  the trace printout */
+		  if (vfd != 0) close(0);
+		  if (vfd != 1) close(1);
+		  vfp = fdopen(vfd, "r");
+		  while (vfp != NULL) {
+		    while (fgets(buf, sizeof buf, vfp) != NULL)
+		      fprintf(stderr, "%s", buf);
+		    if (strncmp(buf, "scheduler done", 14) == 0)
+		      break;
+		    clearerr(vfp);
+		    sleep(1);
+		  }
 		}
 		if (verbfile)
-			unlink(verbfile);
+		  unlink(verbfile);
 		exit(EX_OK);
 		/* NOTREACHED */
 	}
 	if (path != NULL)
-		free(path);
+	  free(path);
 	exit(EX_UNAVAILABLE);
 	/* NOTREACHED */
 	return 0;
