@@ -1072,6 +1072,10 @@ sequencer(e, file)
 			break;
 		}
 	dprintf("It is%s...\n", e->e_resent > 0 ? "" : "n't");
+/* Actually it may be resent, but we (people at UTU.FI) don't like all
+   that it causes latter on.. */
+	e->e_resent = 0;
+
 	dprintf("Sender authentication\n");
 	e->e_trusted = isgoodguy(e->e_statbuf.st_uid);
 	dprintf("Sender is%s trusted\n", e->e_trusted ? "" : "n't");
