@@ -29,10 +29,6 @@
 #include <sys/file.h>
 #endif	/* USE_INET */
 
-#ifdef sgi
-flock() { return 0; }
-#endif
-
 #ifdef	MALLOC_TRACE
 struct conshell *envarlist = NULL;
 #endif	/* MALLOC_TRACE */
@@ -354,7 +350,7 @@ docat(file, fd)
 {
 	char buf[BUFSIZ];
 	int n;
-	FILE *fp;
+	FILE *fp = NULL;
 
 	if (fd < 0 && (fp = fopen(file, "r")) == NULL) {
 	  fprintf(stderr, "%s: %s: %s\n", progname, file, strerror(errno));
@@ -627,9 +623,15 @@ parse(fp)
 	  switch (list) {
 	  case L_CTLFILE:
 	    /* decid:\tfile\tnaddr; off1[,off2,...][\t#message] */
-	    if (!isdigit(buf[0]) && EQNSTR(buf, names[L_CHANNEL])) {
-	      list = L_CHANNEL;
-	      break;
+	    if (!isdigit(buf[0])) {
+	      if (EQNSTR(buf, names[L_CHANNEL])) {
+		list = L_CHANNEL;
+		break;
+	      }
+	      if (EQNSTR(buf, names[L_END])) {
+		list = L_END;
+		break;
+	      }
 	    }
 	    if (!isdigit(buf[0]) ||
 		(cp = strchr(buf, ':')) == NULL) {
