@@ -34,6 +34,12 @@ extern void reporterrs __((struct ctlfile *cfpi));
 extern char *mail_alloc __((u_int n));
 extern int  mail_free __((char *s));
 
+/* pipes.c */
+extern int  pipes_create         __((int *tochild, int *fromchild));
+extern void pipes_close_parent   __((int *tochild, int *fromchild));
+extern void pipes_to_child_fds   __((int *tochild, int *fromchild));
+extern void pipes_shutdown_child __((int fd)); /* At parent, shutdown channel towards child */
+
 /* qprint.c */
 extern void qprint __((int fd));
 
@@ -98,7 +104,8 @@ extern SIGNAL_TYPE sig_chld __((int));
 
 /* update.c */
 extern void update __((int, char *));
-extern void unctlfile __((struct ctlfile *cfp));
+extern void unctlfile __((struct ctlfile *cfp, int no_unlink));
+extern void unvertex __((struct vertex *, int justfree));
 extern void deletemsg __((char *, struct ctlfile *));
 extern char *saytime __((u_long, char *, int));
 extern void expire __((struct vertex *, int));
