@@ -77,8 +77,7 @@ struct threadgroup *thgp;
 if (verbose) printf("delete_threadgroup(%s/%d/%s)\n",
 		    thgp->wchan->name,thgp->withhost,thgp->whost->name);
 
-	if (thgp->idleproc != NULL ||
-	    thgp->threads > 0) abort();
+	if (thgp->idleproc != NULL || thgp->thread != NULL) abort();
 	if (thrg_root == NULL) abort();
 
 
@@ -201,9 +200,8 @@ struct thread *thr;
 	free(thr->channel);
 	free(thr->host);
 
-	if (thr->jobs > 0) {
-	  if (verbose)
-	    printf(" DELETE_THREAD() WITH JOBS=%d\n",thr->jobs);
+	if (thr->jobs != 0) {
+	  printf(" DELETE_THREAD() WITH JOBS=%d\n",thr->jobs);
 	  abort(); /* Delete only when no vertices */
 	}
 
@@ -239,8 +237,12 @@ struct thread *thr;
 		     thr->proc->pid <= 0) {
 	    thr->proc->thread = NULL;
 	    thr->proc->vertex = NULL;
+	    if (verbose)
+	      fprintf(stderr,"delete_thread(1) thr->proc=0x%lx pid=%d\n",(long)thr->proc, thr->proc->pid);
 	  } else {
-	    fprintf(stderr,"delete_thread(1) thr->proc=0x%lx\n",(long)thr->proc);
+	    if (verbose)
+	      fprintf(stderr,"delete_thread(1b) thr->proc=0x%lx pid=%d\n",(long)thr->proc,
+		      thr->proc ? thr->proc->pid : 0);
 	  }
 	} else {
 	  /* Some threads left						*/
@@ -252,14 +254,20 @@ struct thread *thr;
 	    thr->proc->vertex = thg->thread->vertices;
 	    thr->proc->thread = thg->thread;
 	    pick_next_vertex(thr->proc);
+	    if (verbose)
+	      fprintf(stderr, "delete_thread(2) thr->proc=0x%lx pid=%d\n",
+		      (long)thr->proc, thr->proc->pid);
 	  } else if (thr->proc != NULL &&
 		     thr->proc->pid <= 0) {
 	    thr->proc->thread = NULL;
 	    thr->proc->vertex = NULL;
-	    fprintf(stderr, "delete_thread(2) thr->proc=0x%lx pid=%d\n",
-		    (long)thr->proc, thr->proc->pid);
+	    if (verbose)
+	      fprintf(stderr, "delete_thread(2b) thr->proc=0x%lx pid=%d\n",
+		      (long)thr->proc, thr->proc->pid);
 	  } else {
-	    fprintf(stderr, "delete_thread(3) thr->proc=0x%lx\n", (long)thr->proc);
+	    if (verbose)
+	      fprintf(stderr, "delete_thread(3) thr->proc=0x%lx pid=%d\n", (long)thr->proc,
+		      thr->proc ? thr->proc->pid : 0);
 	  }
 	}
 
@@ -714,9 +722,19 @@ struct procinfo *proc;
 if (verbose) printf("pick_next_vertex(proc->tofd=%d, thr=0x%lx, jobs=%d)\n",
 		    proc->tofd, (long)thr, thr ? thr->jobs : 0);
 
+	if (proc->pid < 0) {	/* "Jim, He is dead!"		*/
+	  if (proc->thread != NULL)
+	    proc->thread->proc = NULL;
+	  proc->thread = NULL;
+	  if (proc->vertex != NULL)
+	    proc->vertex->proc = NULL;
+	  proc->vertex = NULL;
+	  return;
+	}
+
 	if (thr == NULL) return; /* WE ARE IDLE! */
 
-	if (proc->fed == 0) return; /* Current one has not been fed.. */
+	if (proc->fed == 0) return; /* Current one has not been fed..	*/
 	proc->fed = 0;
 
 	thr0 = thr;
@@ -848,7 +866,7 @@ reschedule(vp, factor, index)
 	    printf("ce.retries = %d\n", vp->ce.nretries);
 	  return;
 	}
-
+#if 0
 	if (vp->ce.expiry > 0 && vp->ce.expiry <= now && vp->attempts > 0) {
 	  if (verbose)
 	    printf("ce.expiry = %d, %d attempts\n",
@@ -856,12 +874,7 @@ reschedule(vp, factor, index)
 	  expire(vp, index);
 	  return;
 	}
-
-	/* unlink from the list of scheduled vertices */
-	unthread(vp);
-	/* NOW THE THREAD CAN BE WITHOUT ANY VERTEX !
-	   WE MUST LINK IT BACK!                      */
-
+#endif
 	if (factor == -1) {
 	  if (vp->retryindex >= vp->ce.nretries)
 	    vp->retryindex = ranny(vp->ce.nretries-1);
@@ -895,17 +908,18 @@ reschedule(vp, factor, index)
 	  if (verbose)
 	    printf("ce.expiry = %d, %d attempts\n",
 		   (int)(vp->ce.expiry), vp->attempts);
-	  /* XXX: Do we expire correctly ?
-	          Now this vertex is not on any thread! */
-	  expire(vp, index);
 
-	  /* The thread can now be EMPTY! */
-	  if (thr->vertices == NULL)
-	    delete_thread(thr);
+	  /* expire() will delete this vertex in due time */
+	  expire(vp, index);
 
 	  return;
 	}
 
+	/* unlink from the list of scheduled vertices */
+	unthread(vp);
+	/* NOW THE THREAD CAN BE WITHOUT ANY VERTEX ! (thr->jobs == 0 !)
+	   WE MUST LINK IT BACK!                      */
+
 	if (verbose)
 	  printf("wakeup %d pending %d\n", (int)(vp->wakeup), vp->ce.pending);
 
@@ -968,13 +982,16 @@ if (verbose) printf("idle_cleanup()\n");
 	  struct threadgroup *thgn = thg->next;
 	  thg_once = 0;
 
-	  if (thg->idlecnt > 0) {
-
+	  if (thg->idleproc != NULL) {
+	    int idlecnt = 0;
+	    int newidlecnt = 0;
 	    struct procinfo *p, **pp;
+	    
 	    p  =  thg->idleproc;
 	    pp = &thg->idleproc;
 
 	    while (p != NULL) {
+	      ++idlecnt;
 	      if (thg->cep->idlemax + p->hungertime < now) {
 		/* It is old enough -- ancient, one might say.. */
 
@@ -984,26 +1001,28 @@ if (verbose) printf("idle_cleanup()\n");
 		  printf("idle_cleanup() killing TA on tofd=%d pid=%d\n",
 			 p->tofd,p->pid);
 		write(p->tofd,"\n",1);
-		close(p->tofd);
-		p->tofd = -1;
+		pipes_shutdown_child(p->tofd);
+		p->tofd       = -1;
+		p->thg        = NULL;
+		p->thread     = NULL;
 		thg->idlecnt -= 1;
 		idleprocs    -= 1;
 		++freecount;
 		/* The thread-group can be deleted before reclaim() runs! */
 		thg->transporters -= 1;
-		p->thg = NULL;
+		--numkids;
 
-		/* Remove this entry from the chains */
-		*pp = p->next;
-		p = p->next;
+		/* Remove this entry from the chain, and move to a next one */
+		p = *pp = p->next;
 
 	      } else {
+		++newidlecnt;
 		/* Move to the next possible idle process */
 		pp = &p->next;
 		p = p->next;
 	      }
 	    }
-	    if (thg->threads == 0 && thg->idlecnt == 0) {
+	    if (thg->thread == NULL && thg->idleproc == NULL) {
 	      /* No threads, no idle processes! Delete it! */
 	      delete_threadgroup(thg);
 	    }
@@ -1057,6 +1076,10 @@ FILE *fp;
 	    }
 	    if (thr->wakeup > now)
 	      fprintf(fp," wakeup=%d",(int)(thr->wakeup - now));
+	    if (thr->vertices && thr->vertices->ce.pending)
+	      fprintf(fp,
+		      thr->vertices->ce.pending == SIZE_L ? "" :
+		      (thr->vertices->ce.pending == L_CHANNEL ? " channelwait" : " threadwait"));
 	    fprintf(fp,"\n");
 	  }
 	  fprintf(fp,"\t\tThreads: %d/%d ",thg->threads,cnt);
@@ -1065,5 +1088,5 @@ FILE *fp;
 	  for (;p != 0; p = p->next) ++cnt;
 	  fprintf(fp,"Idle-procs: %d/%d\n",thg->idlecnt,cnt);
 	}
-	fprintf(fp,"Idleprocs: %d\n",idleprocs);
+	fprintf(fp,"Idleprocs: %d   NumKids: %d\n",idleprocs, numkids);
 }
