@@ -48,7 +48,6 @@ int	numkids = 0;
 extern char *getzenv();
 extern char *strchr();
 extern int cistrcmp(), ranny();
-extern int epipe();
 extern int errno;
 extern time_t time();
 
@@ -176,29 +175,17 @@ static int runcommand(argv, vhead, chwp, howp)
 	gid = vhead->ce.gid;
 	cmd = vhead->ce.command;
 
-	if (epipe(to) < 0)
-	  return 0;
-	if (epipe(from) < 0) {
-	  close(to[0]);
-	  close(to[1]);
-	  return 0;
-	}
+	if (pipes_create(to,from) < 0) return 0;
 	if (pipesize == 0)
 	  pipesize = resources_query_pipesize(to[0]);
 
 	if (verbose)
 	  fprintf(stderr, "to %d/%d from %d/%d\n",
 		  to[0],to[1],from[0],from[1]);
+
 	if ((pid = fork()) == 0) {	/* child */
-	  if (to[0] != 0) {
-	    dup2(to[0],0);	/* child stdin */
-	    close(to[0]);
-	  }
-	  if (from[1] != 1) {
-	    dup2(from[1],1);	/* child stdout */
-	    close(from[1]);
-	  }
-	  /* dup2(1,2); */
+
+	  pipes_to_child_fds(to,from);
 
 	  /* keep current stderr for child stderr */
 	  /* close all other open filedescriptors */
@@ -225,11 +212,8 @@ static int runcommand(argv, vhead, chwp, howp)
 
 	/* parent */
 
-	close(to[0]);
-	close(from[1]);
-	++numkids;
-	if (chwp != NULL) chwp->kids += 1;
-	if (howp != NULL) howp->kids += 1;
+	pipes_close_parent(to,from);
+
 	/* save from[0] away as a descriptor to watch */
 	stashprocess(pid, from[0], to[1], chwp, howp, vhead);
 	/* We wait for the child to report "#hungry", then we feed it.. */
@@ -268,6 +252,9 @@ static void stashprocess(pid, fromfd, tofd, chwp, howp, vhead)
 	proc->thread->proc = proc;
 	proc->thg    = vhead->thread->thgrp;
 	proc->thg->transporters += 1;
+	++numkids;
+	if (chwp != NULL) chwp->kids += 1;
+	if (howp != NULL) howp->kids += 1;
 	proc->tofd   = tofd;
 	vhead->proc  = proc;
 	proc->carryover = NULL;
@@ -333,6 +320,7 @@ if (verbose)
 	if (proc->thread != NULL) {
 	  nvp = proc->thread->vertices;
 	  proc->thg->transporters -= 1;
+	  --numkids;
 	  proc->vertex = NULL;
 	  proc->thread = NULL;
 	  proc->thg    = NULL;
@@ -344,7 +332,7 @@ if (verbose)
 	    reschedule(vp, 0, -1);
 	  }
 	} else {
-	  /* We were in idle chain! */
+	  /* Maybe we were in idle chain! */
 	  if (proc->thg != NULL /* And not killed by  idle_cleanup() */) {
 	    struct procinfo *p, **pp;
 	    p  =  proc->thg->idleproc;
@@ -359,10 +347,22 @@ if (verbose)
 	      /* Remove this entry from the chains */
 	      *pp = p->next;
 	      p = p->next;
+	      proc->thg->idlecnt      -= 1;
 	      proc->thg->transporters -= 1;
+	      --numkids;
+	      /* It may go down to zero.. */
+	      if (proc->thg->transporters == 0 && proc->thg->threads == 0)
+		delete_threadgroup(proc->thg);
 	    } else {
 	      fprintf(stderr, "reclaim(fd=%d) proc->thg=0x%lx, proc=0x%lx NOT IN IDLE CHAIN!\n", fromfd, (long)proc->thg, (long)proc);
+	      proc->thg->idlecnt      -= 1;
+	      proc->thg->transporters -= 1;
+	      --numkids;
 	    }
+	  } else {
+	    /* We are killed by the idle_cleanup() !   */
+	    /* proc->thg == NULL, proc->thread == NULL */
+	    /* idle_cleanup() did all decrementing..   */
 	  }
 	}
 }
@@ -374,13 +374,13 @@ static void waitandclose(fd)
 	   - fd return 0 (EOF)
 	   - fd returns -1, and errno != {EAGAIN|EWOULDBLOCK|EINTR}
 	 */
-	--numkids;
 	reclaim(fd, cpids[fd].tofd);
 
 }
 
 #ifdef	USE_SELECT
 
+
 #if	defined(BSD4_3) || defined(sun)
 #include <sys/file.h>
 #endif
@@ -416,12 +416,41 @@ typedef	struct fd_set { fd_mask	fds_bits[1]; } fd_set;
 #endif	/* !FD_SET */
 #endif	/* !NFDBITS */
 
+#ifdef _AIX
+#include <sys/types.h>
+#include <sys/select.h>
+	struct {
+		fd_set  fdsmask;
+		fd_set  msgsmask;
+	} readfds, writefds;
+#define _FD_SET(sock,var) FD_SET(sock,&var.fdsmask)
+#define _FD_CLR(sock,var) FD_CLR(sock,&var.fdsmask)
+#define _FD_ZERO(var) FD_ZERO(&var.fdsmask)
+#define _FD_ISSET(i,var) FD_ISSET(i,&var.fdsmask)
+#else
+#ifdef FD_SET
+#define _FD_SET(sock,var) FD_SET(sock,&var)
+#define _FD_CLR(sock,var) FD_CLR(sock,&var)
+#define _FD_ZERO(var) FD_ZERO(&var)
+#define _FD_ISSET(i,var) FD_ISSET(i,&var)
+#else
+#define _FD_SET(sock,var) var |= (1 << sock)
+#define _FD_CLR(sock,var) var &= ~(1 << sock)
+#define _FD_ZERO(var) var = 0
+#define _FD_ISSET(i,var) ((var & (1 << i)) != 0)
+#endif
+#endif
+
 int
 mux(timeout)
 time_t timeout;
 {
 	int	i, n, rc, maxf;
+#ifdef _AIX
+	struct { fd_set fdsmask; fd_set msgsmask; } mask;
+#else
 	fd_set	mask;
+#endif
 	struct sockaddr_in raddr;
 	int	raddrlen;
 	struct timeval tv;
@@ -434,15 +463,15 @@ time_t timeout;
 	tv.tv_usec = 0;
 
 	maxf = -1;
-	FD_ZERO(&mask);
+	_FD_ZERO(mask);
 	if (cpids != NULL)
 	  for (i = 0; i < scheduler_nofiles ; ++i)
 	    if (cpids[i].pid != 0) {
-	      FD_SET(i, &mask);
+	      _FD_SET(i, mask);
 	      maxf = i;
 	    }
 	if (querysocket >= 0) {
-	  FD_SET(querysocket, &mask);
+	  _FD_SET(querysocket, mask);
 	  if (maxf < querysocket)
 	    maxf = querysocket;
 	}
@@ -461,9 +490,9 @@ time_t timeout;
 	  return 1;
 	} else {
 	  /*fprintf(stderr, "got %d ready (%x)\n", n, mask.fds_bits[0]);*/
-	  if (querysocket >= 0 && FD_ISSET(querysocket, &mask)) {
+	  if (querysocket >= 0 && _FD_ISSET(querysocket, mask)) {
 	    --n;
-	    FD_CLR(querysocket, &mask);
+	    _FD_CLR(querysocket, mask);
 	    raddrlen = sizeof raddr;
 	    i = accept(querysocket, (struct sockaddr *)&raddr, &raddrlen);
 	    if (i < 0) {
@@ -473,22 +502,19 @@ time_t timeout;
 	      if (rc == 0) { /* Child! */
 		close(querysocket);
 		qprint(i);
-		/* When running memory leakage debuggers we do not care
-		   that this leaks memory, thus we run /bin/true, and
-		   things like PURIFY and DEC ATOM don't complain.. */
-		execl("/bin/true","true");
+		execl("/bin/false",NULL);
 		_exit(0);
 	      }
-	      if (rc > 0)
-		++numkids;
+	      /* if (rc > 0)
+		 ++numkids; */
 	      close(i);
 	    }
 	  }
 	  if (cpids != NULL)
 	    for (i = 0; i < maxf; ++i) {
-	      if (cpids[i].pid != 0 && FD_ISSET(i, &mask)) {
+	      if (cpids[i].pid != 0 && _FD_ISSET(i, mask)) {
 		--n;
-		FD_CLR(i, &mask);
+		_FD_CLR(i, mask);
 		/*fprintf(stderr,"that is fd %d\n",i);*/
 		/* do non-blocking reads from this fd */
 		readfrom(i);
@@ -527,13 +553,13 @@ queryipcinit()
 	setsockopt(querysocket, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));
 
 	if (bind(querysocket, (struct sockaddr *)&sad, sizeof sad) < 0) {
-	  perror("bind");
+	  perror("bind:mailq socket");
 	  close(querysocket);
 	  querysocket = -1;
 	  return;
 	}
 	if (listen(querysocket, 5) < 0) {
-	  perror("listen");
+	  perror("listen:mailq socket");
 	  close(querysocket);
 	  querysocket = -1;
 	  return;
@@ -773,7 +799,6 @@ int signum;
 #endif
 	  if (pid <= 0) return;
 
-
 #ifdef	WIFEXITED
 	  if (WIFEXITED(statloc)) ok = 1;
 #else
@@ -794,12 +819,14 @@ int signum;
 	    for (i = scheduler_nofiles-1; i >= 0; --i) {
 	      if (cpids[i].pid == pid) {
 		cpids[i].pid = -pid; /* Mark it as reaped.. */
+		cpids[i].reaped = 1;
 		ok = 0;
 		break;
 	      }
 	      if (cpids[i].pid == -pid) {
 		printf(" .. already reaped ??\n");
 		cpids[i].pid = -pid; /* Mark it as reaped.. */
+		cpids[i].reaped = 1;
 		ok = 0;
 		break;
 	      }
