@@ -3,7 +3,6 @@
  *	This will be free software, but only when it is finished.
  */
 
-
 #include <stdio.h>
 #include <sys/param.h>
 #include "hostenv.h"
@@ -489,12 +488,41 @@ typedef	struct fd_set { fd_mask	fds_bits[1]; } fd_set;
 #endif	/* !FD_SET */
 #endif	/* !NFDBITS */
 
+#ifdef _AIX
+#include <sys/types.h>
+#include <sys/select.h>
+	struct {
+		fd_set  fdsmask;
+		fd_set  msgsmask;
+	} readfds, writefds;
+#define _FD_SET(sock,var) FD_SET(sock,&var.fdsmask)
+#define _FD_CLR(sock,var) FD_CLR(sock,&var.fdsmask)
+#define _FD_ZERO(var) FD_ZERO(&var.fdsmask)
+#define _FD_ISSET(i,var) FD_ISSET(i,&var.fdsmask)
+#else
+#ifdef FD_SET
+#define _FD_SET(sock,var) FD_SET(sock,&var)
+#define _FD_CLR(sock,var) FD_CLR(sock,&var)
+#define _FD_ZERO(var) FD_ZERO(&var)
+#define _FD_ISSET(i,var) FD_ISSET(i,&var)
+#else
+#define _FD_SET(sock,var) var |= (1 << sock)
+#define _FD_CLR(sock,var) var &= ~(1 << sock)
+#define _FD_ZERO(var) var = 0
+#define _FD_ISSET(i,var) ((var & (1 << i)) != 0)
+#endif
+#endif
+
 int
 mux(timeout)
 time_t timeout;
 {
 	int	i, n, rc, maxf;
+#ifdef _AIX
+	struct { fd_set fdsmask; fd_set msgsmask; } mask;
+#else
 	fd_set	mask;
+#endif
 	struct sockaddr_in raddr;
 	int	raddrlen;
 	struct timeval tv;
@@ -507,15 +535,15 @@ time_t timeout;
 	tv.tv_usec = 0;
 
 	maxf = -1;
-	FD_ZERO(&mask);
+	_FD_ZERO(mask);
 	if (cpids != NULL)
 	  for (i = 0; i < scheduler_nofiles ; ++i)
 	    if (cpids[i].pid != 0) {
-	      FD_SET(i, &mask);
+	      _FD_SET(i, mask);
 	      maxf = i;
 	    }
 	if (querysocket >= 0) {
-	  FD_SET(querysocket, &mask);
+	  _FD_SET(querysocket, mask);
 	  if (maxf < querysocket)
 	    maxf = querysocket;
 	}
@@ -534,9 +562,9 @@ time_t timeout;
 	  return 1;
 	} else {
 	  /*fprintf(stderr, "got %d ready (%x)\n", n, mask.fds_bits[0]);*/
-	  if (querysocket >= 0 && FD_ISSET(querysocket, &mask)) {
+	  if (querysocket >= 0 && _FD_ISSET(querysocket, mask)) {
 	    --n;
-	    FD_CLR(querysocket, &mask);
+	    _FD_CLR(querysocket, mask);
 	    raddrlen = sizeof raddr;
 	    i = accept(querysocket, (struct sockaddr *)&raddr, &raddrlen);
 	    if (i < 0) {
@@ -556,9 +584,9 @@ time_t timeout;
 	  }
 	  if (cpids != NULL)
 	    for (i = 0; i < maxf; ++i) {
-	      if (cpids[i].pid != 0 && FD_ISSET(i, &mask)) {
+	      if (cpids[i].pid != 0 && _FD_ISSET(i, mask)) {
 		--n;
-		FD_CLR(i, &mask);
+		_FD_CLR(i, mask);
 		/*fprintf(stderr,"that is fd %d\n",i);*/
 		/* do non-blocking reads from this fd */
 		readfrom(i);
