@@ -56,7 +56,7 @@ main(argc, argv)
 	int argc;
 	char *argv[];
 {
-	char *channel, file[MAXPATHLEN+1];
+	char *channel, msgfilename[MAXPATHLEN+1];
 	int errflg, c;
 	struct ctldesc *dp;
 	SIGNAL_TYPE (*oldsig)();
@@ -103,27 +103,26 @@ main(argc, argv)
 
 	  printf("#hungry\n");
 	  fflush(stdout);
-	  if (fgets(file, sizeof file, stdin) == NULL)
-	    break;
+
+	  if (fgets(msgfilename, sizeof msgfilename, stdin) == NULL) break;
+
 	  /* Input:
 	       spool/file/name [ \t host.info ] \n
 	   */
 
-	  if (emptyline(file, sizeof file))
+	  if (emptyline(msgfilename, sizeof msgfilename))
 	    break;
 
-	  s = strchr(file,'\t');
+	  s = strchr(msgfilename,'\t');
 	  if (s != NULL)
 	    *s = 0;	/* Ignore the host-selector */
 
-	  dp = ctlopen(file, channel, (char *)NULL, &getout, ctlsticky, NULL);
+	  dp = ctlopen(msgfilename, channel, (char *)NULL, &getout, ctlsticky, NULL);
 	  if (dp != NULL) {
 	    process(dp);
 	    ctlclose(dp);
 	  }
 	}
-	exit(0);
-	/* NOTREACHED */
 	return 0;
 }
 
