@@ -342,11 +342,11 @@ main(argc, argv)
 	    creatflag = !creatflag;
 	    break;
 	  case 'h':
-	    host = strsave(optarg);
+	    host = strdup(optarg);
 	    matchhost = 1;
 	    break;
 	  case 'l':		/* log file */
-	    logfile = strsave(optarg);
+	    logfile = strdup(optarg);
 	    break;
 	  case '8':
 	    mime8bit = 1;
@@ -398,7 +398,7 @@ main(argc, argv)
 	  s = strchr(file,'\t');
 	  if (s != NULL) {
 	    if (host) free(host);
-	    host = strsave(s+1);
+	    host = strdup(s+1);
 	    *s = 0;
 	  }
 
@@ -482,7 +482,7 @@ process(dp)
 	   * "contents" will be freed before rbiff done:
 	   * must copy username somewhere.
 	   */
-	  rp->addr->user = strsave(rp->addr->user);
+	  rp->addr->user = strdup(rp->addr->user);
 #endif /* RBIFF */
 	}
 
@@ -611,7 +611,7 @@ deliver(dp, rp, timestring)
 	    setrootuid(rp);
 	    return;
 	  }
-	  file = strsave(rp->addr->user);
+	  file = strdup(rp->addr->user);
 	  if (access(file, F_OK) < 0) {
 	      fdmail = createfile(rp, file, uid);
 	      if (fdmail < 0) {
@@ -695,7 +695,7 @@ deliver(dp, rp, timestring)
 	    break;
 	  nbp = (struct biffer *)emalloc(sizeof (struct biffer));
 	  nbp->next = NULL;
-	  nbp->user = strsave(rp->addr->user);
+	  nbp->user = strdup(rp->addr->user);
 	  nbp->offset = -1;
 	  nbp->wantrbiff = 0;
 #ifdef	RBIFF
@@ -1825,7 +1825,7 @@ readrwho()
 
 	    uhp = (struct userhost *)emalloc(sizeof (struct userhost));
 	    uhp->next = (struct userhost *)spl->data;
-	    uhp->hostname = strsave(w->wd_hostname);
+	    uhp->hostname = strdup(w->wd_hostname);
 	    spl->data = (unsigned char *)uhp;
 	  }
 	  close(f);
