@@ -229,8 +229,10 @@ extern void warning(), report();
 #ifndef MALLOC_TRACE
 extern univptr_t emalloc();
 #endif
-extern int  atoi __((char*));
-extern long atol __((char*));
+/*
+   extern int  atoi __((char*));
+   extern long atol __((char*));
+ */
 extern char *strerror();
 extern char *strchr();
 extern char *strrchr();
@@ -474,6 +476,7 @@ main(argc, argv)
 	    *s++ = 0;
 	    if (host && strcmp(s,host) != 0) {
 	      if (smtp_isopen) {
+		smtpstatus = smtpwrite(smtpfp, verboselog, 0, "QUIT");
 		fclose(smtpfp[0]); smtpfp[0] = NULL;
 		fclose(smtpfp[0]); smtpfp[0] = NULL;
 		if (logfp)
@@ -925,7 +928,7 @@ appendlet(dp, fp, verboselog, hsize, msize, convertmode)
 	    }
 	    return EX_IOERR;
 	  }
-	  lastwasnl = (let_buffer[readalready-1] != '\n')
+	  lastwasnl = (let_buffer[readalready-1] != '\n');
 	  if (statusreport)
 	    report("DATA %d (100%%)", readalready+hsize);
 	  if (fflush(fp) != 0)
@@ -1641,7 +1644,7 @@ makeconn(hp, fdp, verboselog, fparr, mxcnt)
 	    break;		/* TEMPFAIL or UNAVAILABLE.. */
 	  }
 
-	  if (fparr[0] != NULL || fparr[1] != NULL) {
+	  if (smtp_isopen) {
 	    /* Clean (close) these fds -- they have noted to leak.. */
 	    if (fparr[0] != NULL) fclose(fparr[0]); fparr[0] = NULL;
 	    if (fparr[1] != NULL) fclose(fparr[1]); fparr[1] = NULL;
@@ -1665,6 +1668,7 @@ fdopen_fail:
 	      if ((fparr[1] = fdopen(*fdp, "w")) == NULL) {
 		goto fdopen_fail;
 	      }
+	      smtp_isopen = 1;
 
 	      if (esmtp_on_banner > 0)
 		esmtp_on_banner = 0;
@@ -1903,9 +1907,9 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	      fprintf(verboselog,"%s\n",remotemsg);
 	    fclose(smtpfp[0]); smtpfp[0] = NULL;
 	    fclose(smtpfp[1]); smtpfp[1] = NULL;
+	    smtp_isopen = 0;
 	    if (logfp)
 	      fprintf(logfp, "%d#\t(closed SMTP channel - timeout on smtpwrite())\n", pid);
-	    smtp_isopen = 0;
 	    return EX_TEMPFAIL;
 	  } else if (r != len) {
 	    sprintf(remotemsg, "500 (SMTP cmd write failure: Only wrote %d of %d bytes!)", r, len);
@@ -1914,9 +1918,9 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	      fprintf(verboselog,"%s\n",remotemsg);
 	    fclose(smtpfp[0]); smtpfp[0] = NULL;
 	    fclose(smtpfp[1]); smtpfp[1] = NULL;
+	    smtp_isopen = 0;
 	    if (logfp)
 	      fprintf(logfp, "%d#\t(closed SMTP channel - second timeout on smtpwrite() )\n", pid);
-	    smtp_isopen = 0;
 	    return EX_TEMPFAIL;
 	  }
 	  if (logfp != NULL) {
@@ -2036,9 +2040,9 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	      fprintf(verboselog,"%s\n",remotemsg);
 	    fclose(smtpfp[0]); smtpfp[0] = NULL;
 	    fclose(smtpfp[1]); smtpfp[1] = NULL;
+	    smtp_isopen = 0;
 	    if (logfp)
 	      fprintf(logfp, "%d#\t(closed SMTP channel - bad responce on smtpwrite() )\n", pid);
-	    smtp_isopen = 0;
 	    return EX_TEMPFAIL;
 	  } else {
 	    sprintf(remotemsg, "500 (Server hung up on us! Cmd: %s)",
@@ -2049,9 +2053,9 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	      fprintf(verboselog,"%s\n",remotemsg);
 	    fclose(smtpfp[0]); smtpfp[0] = NULL;
 	    fclose(smtpfp[1]); smtpfp[1] = NULL;
+	    smtp_isopen = 0;
 	    if (logfp)
 	      fprintf(logfp, "%d#\t(closed SMTP channel - hangup on smtpwrite() )\n", pid);
-	    smtp_isopen = 0;
 	    return EX_TEMPFAIL;
 	  }
 	  /* Exit if the last thing we read was a LF and we're on the
@@ -2067,9 +2071,9 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	    fprintf(verboselog,"%s\n",remotemsg);
 	  fclose(smtpfp[0]); smtpfp[0] = NULL;
 	  fclose(smtpfp[1]); smtpfp[1] = NULL;
+	  smtp_isopen = 0;
 	  if (logfp)
 	    fprintf(logfp, "%d#\t(closed SMTP channel - responce overrun on smtpwrite() )\n", pid);
-	  smtp_isopen = 0;
 	  return EX_TEMPFAIL;
 	}
 	*--cp = '\0';	/* kill the LF */
@@ -2081,9 +2085,9 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	    fprintf(verboselog,"%s\n",remotemsg);
 	  fclose(smtpfp[0]); smtpfp[0] = NULL;
 	  fclose(smtpfp[1]); smtpfp[1] = NULL;
+	  smtp_isopen = 0;
 	  if (logfp)
 	    fprintf(logfp, "%d#\t(closed SMTP channel - unexpected responce on smtpwrite() )\n", pid);
-	  smtp_isopen = 0;
 	  return EX_TEMPFAIL;
 	}
 	--cp;
